/*********************************************************************
*                SEGGER MICROCONTROLLER GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 2003-2014     SEGGER Microcontroller GmbH & Co KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : Performance.c
Purpose : USB BULK performance test application
---------------------------END-OF-HEADER------------------------------
*/

#include <stdio.h>
#include <conio.h>
#include <windows.h>
#include "USBBULK.h"

/*********************************************************************
*
*       defines, configurable
*
**********************************************************************
*/
#define CMD_TEST_SPEED         0x01
#define CMD_TEST               0x02
#define SUBCMD_SPEED_READ             100
#define SUBCMD_SPEED_WRITE            101

#define MAX_NUM_BYTES_TEST_NET    (4100)

/*********************************************************************
*
*       static data
*
**********************************************************************
*/
static HWND _hWnd = INVALID_HANDLE_VALUE;

/*********************************************************************
*
*       static code
*
**********************************************************************
*/

/*********************************************************************
*
*       USB__StoreU32LE
*/
static void _StoreU32LE(U8 * p, U32 v) {
  *p       = (U8)((v      ) & 255);
  *(p + 1) = (U8)((v >>  8) & 255);
  *(p + 2) = (U8)((v >> 16) & 255);
  *(p + 3) = (U8)((v >> 24) & 255);
}

/*********************************************************************
*
*       USB__StoreU16LE
*/
static void _StoreU16LE(U8 * p, unsigned v) {
  *p       = (U8)((v      ) & 255);
  *(p + 1) = (U8)((v >>  8) & 255);
}


/*********************************************************************
*
*       _MessageBox
*
*/
static void _MessageBox(const char * s) {

  if (_hWnd == INVALID_HANDLE_VALUE) {
    char ac[200];
    GetConsoleTitle(ac, sizeof(ac));
    _hWnd = FindWindow("ConsoleWindowClass", ac);
  }
  MessageBox(_hWnd, s, "USB Bulk sample application", MB_OK | MB_APPLMODAL);
}


/*********************************************************************
*
*       _Receive
*
*/
static int _Receive(unsigned DevIndex, U8 * pBuffer, U32 NumBytes2Read) {
  int r;
  r = USBBULK_Read(pBuffer, NumBytes2Read);
  if (r < 1) {
    _MessageBox("Could not read from device");
    return 1;                      // Error
  }
  printf(".");
  return 0;
}

/*********************************************************************
*
*       _Send
*
*/
static int _Send(unsigned DevIndex, U8 * pBuffer, U32 NumBytes2Write) {
  int r;
  r = USBBULK_WriteEx(DevIndex, pBuffer, NumBytes2Write);
  if (r < 1) {
    _MessageBox("Could not write to device");
    return 1;                      // Error
  }
  printf(".");
  return 0;
}


/*********************************************************************
*
*       _ShowDriverInfo
*
*/
static void _ShowDriverInfo(void) {
  char ac[200];
  unsigned Ver;
  USBBULK_GetDriverCompileDate(ac, sizeof(ac));
  Ver = USBBULK_GetDriverVersion();
  printf("USB BULK driver version: %d.%.2d%c, compiled: %s\n", Ver / 10000, (Ver / 100) % 100, (Ver % 100) + 'a', ac);
}

/*********************************************************************
*
*       _GetNumberFromConsole
*
*/
static void _GetNumberFromConsole(const char * s, unsigned * pNumber) {
  int  r;
  char ac[100];

  printf(s);
  ac[0] = 96;
  _cgets(ac);
  r = atoi(&ac[2]);
  if (r > 0) {
    *pNumber = r;
  }
}

/*********************************************************************
*
*       _FillBuffer
*
*/
static void _FillBuffer(unsigned char * pBuffer, unsigned NumBytes) {
  unsigned i;
  for (i = 0; i < NumBytes; i++) {
    *(pBuffer + i) = (unsigned char )(i % 255);
  }
}


/*********************************************************************
*
*       _TestSpeed
*/
int _TestSpeed(int DevIndex, U8 SubCmd, U32 NumReps, U32 NumBytes) {
  U8  abOut[0x100];
  U8* pData;
  U8* p;
  int NumWords;
  int r = -1;
  int i;

  if ((NumReps | NumBytes) == 0) {
    return 0;
  }
  //
  // Alloc memory
  //
  pData = (U8*)malloc(NumBytes);
  if (pData == NULL) {
    return -1;                        // Error, no memory
  }
  memset(pData, 0, NumBytes);
  NumWords = NumBytes >> 2;
  for (i = 0; i < NumWords; i++) {
    *(U32*)(pData + i * 4) = i;
  }
  //
  // Compose command
  //
  p = &abOut[0];
  *p++ = CMD_TEST_SPEED;
  *p++ = SubCmd;
  _StoreU32LE(p, NumReps);
  _StoreU32LE(p + 4, NumBytes);
  //
  // Transfer command
  //
  if (USBBULK_WriteEx(DevIndex, &abOut[0], 10) != 10) {
    goto CleanUp;                     // Communication error
  }
  //
  // Transfer the data
  //
  do {
    switch (SubCmd) {
    case SUBCMD_SPEED_WRITE:
      if (USBBULK_WriteEx(DevIndex,pData, NumBytes) != (int)NumBytes) {
        goto CleanUp;                 // Communication error
      }
      break;
    case SUBCMD_SPEED_READ:
      if (USBBULK_ReadEx(DevIndex, pData, NumBytes) != (int)NumBytes) {
        goto CleanUp;                 // Communication error
      }
      break;
    }
  } while (--NumReps);
  r = 0;
CleanUp:
  free(pData);
  return r;
}


/*********************************************************************
*
*       _PerformanceTest
*
*/
static int _PerformanceTest(int DevIndex) {
  unsigned   NumBytes = 0x2000;
  unsigned   NumReps  = 0x200;
  U32        t, tWrite, tRead;
  int        r = 0;

  if (USBBULK_OpenEx(DevIndex) == NULL) {
    _MessageBox("Unable to connect to USB BULK device");
    return 1;
  }
  _ShowDriverInfo();
  USBBULK_SetTimeoutEx(DevIndex, 3600 * 1000);
  _GetNumberFromConsole("Enter the packet size in bytes (default: 0x2000): ", &NumBytes);
  _GetNumberFromConsole("Enter the number of packets    (default: 512): ", &NumReps);
  printf("\n Transferring %d KB (%d * %d KB)\n", (NumReps * NumBytes) >> 10, NumReps, NumBytes >> 10);
  t = timeGetTime();
  _TestSpeed(DevIndex, SUBCMD_SPEED_WRITE, NumReps, NumBytes);
  tWrite = timeGetTime() - t;
  if (tWrite < (int)((NumBytes * NumReps) / 50000)) {
    printf("WARNING: Measured speed is too high (> 50 MB/s). Maybe an error occurred during speed test.\n");
  } else if (r < 0) {
    printf("ERROR: Failed to test write speed.\n");
  } else {
    printf("%d KB written in %dms ! (%.1f KB/s)\n", (NumReps * NumBytes) >> 10, tWrite, (float)(NumReps * NumBytes) / tWrite);
  }
  if (r) {
    goto End;
  }
  //
  // Prepare read command and send to device
  //
  t = timeGetTime();
  r = _TestSpeed(DevIndex, SUBCMD_SPEED_READ, NumReps, NumBytes);
  tRead = timeGetTime() - t;
  if (tRead < (int)((NumBytes * NumReps) / 50000)) {
    printf("WARNING: Measured speed is too high (> 50 MB/s). Maybe an error occurred during speed test.\n");
  } else if (r) {
    printf("ERROR: Failed to test read speed.\n");
  } else {
    printf("%d KB read in %dms ! (%.1f KB/s)\n", (NumReps * NumBytes) >> 10, tRead, (float)(NumReps * NumBytes) / tRead);
  }

End:
  USBBULK_CloseEx(DevIndex);
  return r;
}

/*********************************************************************
*
*       _TestNet
*/
static int _TestNet(int DevIndex, int SizeInc, int DelayInc, int DelayMax) {
  U32 AllocSize;
  U8* pWrite;
  U8* pRead;
  U32 NumBytes;
  U32 NumBytesRead;
  int Delay;
  int i;
  int r = -1;

  if (USBBULK_OpenEx(DevIndex) == NULL) {
    _MessageBox("Unable to connect to USB BULK device");
    return 1;
  }
  //
  // Allocate memory
  //
  AllocSize = (MAX_NUM_BYTES_TEST_NET * 2) + 16;
  pWrite = malloc(AllocSize);
  if (pWrite == NULL) {
    printf("ERROR: Could not alloc memory.\n");
    return -1;        // Error
  }
  pRead = pWrite + (AllocSize / 2);
  //
  // Fill memory with test pattern
  //
  for (i = 0; i < MAX_NUM_BYTES_TEST_NET; i++) {
    *(pWrite + i + 4) = i % 255;
  }
  //
  // Perform test in a loop
  //
  for (Delay = 0; Delay <= DelayMax; Delay += DelayInc) {
    for (NumBytes = 1; NumBytes <= MAX_NUM_BYTES_TEST_NET; NumBytes += SizeInc) {
      if (Delay || (NumBytes > 1)) {
        printf("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b");
      }
      printf("  %4d bytes with delay %2d", NumBytes, Delay);
      *(pWrite + 0) = CMD_TEST;
      *(pWrite + 1) = Delay;
      *(pWrite + 2) = (U8)((NumBytes >> 0) & 0xFF);
      *(pWrite + 3) = (U8)((NumBytes >> 8) & 0xFF);
      NumBytesRead = USBBULK_WriteRead(pWrite, (NumBytes + 4), pRead, (NumBytes + 1));
      if (NumBytesRead != (NumBytes + 1)) {
        printf("ERROR: Communication error (Expected %d bytes, received %d)!\n", (NumBytes + 1), NumBytesRead);
        goto Done;
      }
      if (*(pRead + NumBytes) != 0) {
        printf("ERROR: Failed to transfer data!\n");
        goto Done;
      }
      if (memcmp(pWrite + 4, pRead, NumBytes) != 0) {
        printf("ERROR: Verification of data failed!\n");
        goto Done;
      }
      if (_kbhit() != 0) {
        _getch();
        printf("\nTest canceled by user!\n");
        goto Done;    // Canceled by user
      }
    }
  }
  printf("\nTest completed successfully!\n");
  r = 0;              // O.K.
  //
  // Free the allocated memory
  //
Done:
  if (pWrite) {
    free(pWrite);
  }
  return r;
}


/*********************************************************************
*
*       public code
*
**********************************************************************
*/

/*********************************************************************
*
*       main
*
*/
int main(int argc, char* argv[]) {
  int      r;
  U32      DeviceMask;
  char     Restart;
  unsigned NumDevices;
  unsigned DeviceId;
  unsigned Choice;
  char     ChangeTransferSize;
  U32      TransferSize;

  Restart = 'N';
  timeBeginPeriod(1);
  USBBULK_Init();
  do {
    char acRepeat[10];

    DeviceId   = 0;
    NumDevices = USBBULK_GetNumAvailableDevices(&DeviceMask);
    if (NumDevices) {
      printf("Serial no of \n");
      for (r = 0; r < 32; r++) {
        char ac[255];
        if ((1 << r) & DeviceMask) {
          USBBULK_GetSN(r, ac, sizeof(ac));
          printf("Device %d: %s\n", r, ac);
        }
      }
    } else {
      _MessageBox("No devices are available.\n");
      r = 1;
      break;
    }
    printf("To which device do you want to connect? ");
    scanf("%d", &DeviceId);
    if (DeviceId < USB_MAX_DEVICES) {
      if ((1 << DeviceId) & DeviceMask) {
        printf("Current Read  transfer size down is %d\n", USBBULK_GetReadMaxTransferSizeDown(DeviceId));
        printf("Current Write transfer size down is %d\n", USBBULK_GetWriteMaxTransferSizeDown(DeviceId));
        printf("Do you want to change these (y/n)?\n");
        printf("Your choice: ");
        scanf("%c", &Choice);
        acRepeat[0] = 6;
        _cgets(acRepeat);
        ChangeTransferSize = toupper(acRepeat[2]);
        if ((ChangeTransferSize != 'Y') && (ChangeTransferSize != 'N')) {
          ChangeTransferSize = 'N';
        }
        if (ChangeTransferSize == 'Y') {
          printf("Enter the new Read transfer size down size: ");
          scanf("%d", &TransferSize);
          USBBULK_SetReadMaxTransferSizeDown(DeviceId, TransferSize);
          printf("Enter the new Write transfer size down size: ");
          scanf("%d", &TransferSize);
          USBBULK_SetWriteMaxTransferSizeDown(DeviceId, TransferSize);               
        }
        printf("What kind of test would you like to run?\n");
        printf("(1)Performance test\n");
        printf("(2)Stability test\n");
        printf("Your choice: ");
        scanf("%d", &Choice);
        switch (Choice) {
        case 1:
        printf("Starting performance test...\n");

        r = _PerformanceTest(DeviceId);
        if (r) {      
          goto End;
        }
        break;
        case 2:
          printf("Performing quick test...\n");
          r = _TestNet(DeviceId, 47, 5, 5);
          if (r == 0) {
            printf("Performing intensive test...\n");
            r = _TestNet(DeviceId, 1, 2, 10);
          }
          if (r) {      
            goto End;
          }
          break;
        }
        printf("\nStart again? (y/n): ");
        acRepeat[0] = 6;
        _cgets(acRepeat);
        Restart = toupper(acRepeat[2]);
        if ((Restart != 'Y') && (Restart != 'N')) {
          Restart = 'Y';
        }
      } else {
        printf("Wrong device id selected\n");
        Restart = 'Y';
      }
    } 
  } while (Restart == 'Y');  
  if (r == 0) {
    printf("Communication with an USB BULK device was successful!");
  }
End:
  timeEndPeriod(1);
  return r;
}

/******************************* End of file ************************/